@@ -19,8 +19,7 @@ from os import listdir
 from os.path import splitext
 from re import findall, match, sub
 
-from flasker import current_project as pj
-from flasker.project import Project, ProjectImportError
+from flasker.project import current_project as pj, Project, ProjectImportError
 
 
 def _project_context(handler):
