@@ -56,15 +56,14 @@ for :class:`flasker.ext.api.BaseView` for the list of all available options.
 """
 
 from flask import Blueprint, jsonify, request
-from flasker.util import make_view, View as _View, _ViewMeta
 from os.path import abspath, dirname, join
-from sqlalchemy.ext.associationproxy import AssociationProxy
-from sqlalchemy.orm import class_mapper, mapperlib
+from sqlalchemy.orm import class_mapper
 from time import time
 from werkzeug.exceptions import HTTPException
 
-from .orm import BaseModel, Query
-from ..util import uncamelcase, query_to_models
+from ..util.flask import make_view, View as _View, _ViewMeta
+from ..util.helpers import uncamelcase
+from ..util.sqlalchemy import Model, Query, query_to_models
 
 
 class APIError(HTTPException):
@@ -126,7 +125,6 @@ class API(object):
     def api_after_imports(project):
 
       if index_view:
-
         @blueprint.route('/')
         def index():
           return jsonify({
@@ -145,13 +143,24 @@ class API(object):
 
 class _ApiViewMeta(_ViewMeta):
 
-  """To register classes with the API on definition."""
+  """To register classes with the API on definition.
+
+  Automatically creates the ``endpoint``, ``base_url`` and ``rules`` for the
+  view from the ``__model__`` attribute.
+
+  Each route is then registered on the bound application (the current API
+  blueprint here).
+  
+  """
 
   def __new__(cls, name, bases, dct):
 
     model = dct.get('__model__', None)
 
     if model is not None:
+      if not issubclass(model, Model):
+        raise ValueError('Api views can only be used with Orm models.')
+
       dct.setdefault('endpoint', model.__tablename__)
       base_url = dct.setdefault('base_url', model.__tablename__)
 
@@ -199,19 +208,17 @@ class View(_View):
   subviews = []
 
   @classmethod
-  def bind_view(cls, blueprint):
+  def register_view(cls, blueprint):
     """Create the URL routes for the view.
     
     Standard :class:`flasker.util.View` implementation plus subview support.
     
     """
 
-    super(View, cls).bind_view(blueprint)
+    super(View, cls).register_view(blueprint)
 
     if cls.subviews:
-      
       model = cls.__model__
-
       all_keys = set(
         model._get_relationships(
           lazy=['dynamic', True, 'select'],
@@ -229,7 +236,6 @@ class View(_View):
         keys = all_keys & keys
 
       for key in keys:
-
         collection_route = '/%s%s/%s' % (
           cls.base_url,
           ''.join(
@@ -244,13 +250,12 @@ class View(_View):
           ),
           key
         )
-
         make_view(
           blueprint,
           view_class=_RelationshipView,
           view_name='%s_%s' % (cls.endpoint, key),
-          parent_model=model,
-          assoc_key=key,
+          __model__=model,
+          __assoc_key__=key,
           parser=cls.parser,
           endpoint='%s_%s' % (cls.endpoint, key),
           methods=['GET', ],
@@ -262,37 +267,40 @@ class View(_View):
 
   def get(self, **kwargs):
     """GET request handler."""
-    query = self.__model__.q
-    col, match = self.parser.parse(query, model_id=kwargs)
-    return self.jsonify(
-      self.parser.serialize(col),
-      match=match
-    )
+    if kwargs:
+      model = self.__model__.retrieve(from_key=True, **kwargs)
+      if not model:
+        raise APIError(404, 'Not found')
+      return self.parser.jsonify(model)
+    else:
+      return self.parser.jsonify(self.__model__.q)
 
   def post(self):
     """POST request handler."""
     if not self.validate(json):
       raise APIError(400, 'Invalid POST parameters')
     model = self.__model__(**request.json)
-    model._flush()
-    return self.jsonify(self.parser.serialize([model]))
+    model.flush()
+    return self.parser.jsonify(model)
 
   def put(self, **kwargs):
     """PUT request handler."""
-    query = self.__model__.q
-    model = self.parser.parse(query, model_id=kwargs, expect_one=True)
+    model = self.__model__.retrieve(from_key=True, **kwargs)
+    if not model:
+      raise APIError(404, 'Not found')
     if not self.validate(json, model):
       raise APIError(400, 'Invalid PUT parameters')
     for k, v in request.json.items():
       setattr(model, k, v)
-    return self.jsonify(self.parser.serialize([model]))
+    return self.parser.jsonify(model)
 
   def delete(self, **kwargs):
     """DELETE request handler."""
-    query = self.__model__.q
-    model = self.parser.parse(query, model_id=kwargs, expect_one=True)
+    model = self.__model__.retrieve(from_key=True, **kwargs)
+    if not model:
+      raise APIError(404, 'Not found')
     pj.session.delete(model)
-    return self.jsonify(self.parser.serialize([model]))
+    return self.parser.jsonify(model)
 
   def validate(self, json, model=None):
     """Validation method.
@@ -310,26 +318,36 @@ class View(_View):
 
     """
     return True
-
+  
 
 class _RelationshipView(_View):
 
   """Relationship View."""
 
-  parent_model = None
-  assoc_key = None
+  __model__ = None
+  __assoc_key__ = None
 
   def get(self, **kwargs):
     """GET request handler."""
     position = kwargs.pop('position', None)
-    parent = self.parser.parse(
-      self.parent_model.q, model_id=kwargs, expect_one=True
-    )
-    collection =  getattr(parent, self.assoc_key)
-    col, match = self.parser.parse(
-      collection, model_position=position, fast_count=False
-    )
-    return self.jsonify(self.parser.serialize(col), match=match)
+    parent = self.__model__.retrieve(from_key=True, **kwargs)
+    if not parent:
+      raise APIError(404, 'Parent not found')
+    collection =  getattr(parent, self.__assoc_key__)
+
+    if position:
+      position = int(position) - 1 # model_position is 1 indexed
+      if isinstance(collection, Query):
+        model = collection.offset(position).limit(1).first()
+      else:
+        collection = collection[position:(position + 1)]
+        model = collection[0] if collection else None
+        if not model:
+          raise APIError(404, 'Not found')
+      return self.parser.jsonify(model)
+
+    else:
+      return self.parser.jsonify(collection)
 
 
 class Parser(object):
@@ -339,208 +357,165 @@ class Parser(object):
   :param default_depth: the default depth models are jsonified to. ``0`` yields 
     an empty dictionary
   :type default_depth: int
+  :param max_depth: the maximum depth allowed in a query. ``0`` means no limit.
+  :type max_depth: int
   :param default_limit: the default number of results returned per query 
   :type default_limit: int
   :param max_limit: the maximum number of results returned by a query. ``0`` 
     means no limit.
   :type max_limit: int
-  :param expand: if ``True``, same model data will be repeated in the response
-    if the model is encountered multiple times, otherwise only the key will
-    be returned. This can be very useful for efficiency when used with a 
-    client-side library such as Backbone-Relational.
-  :type expand: bool
   :param sep: the separator used for filters and sort parameters
   :type sep: str
+
+  This class has a single method :meth:``jsonify`` which is used to parse a
+  model or collection and return the serialized response.
   
   """
 
   def __init__(self, default_depth=1, max_depth=0, default_limit=20,
-               max_limit=0, expand=True, sep=';'):
+               max_limit=0, sep=';'):
     self.options = {
       'default_depth': default_depth,
       'max_depth': max_depth,
       'default_limit': default_limit,
       'max_limit': max_limit,
-      'expand': expand,
       'sep': sep,
     }
 
-  def parse(self, collection, model_id=None, model_position=None,
-              fast_count=True, expect_one=False):
-    """Parse query and return JSON.
+  def jsonify(self, data, data_key='data', meta_key='meta',
+    include_request=True, include_time=True, include_matches=True, **kwargs):
+    """Parses the data and returns the serialized response.
 
-    :param collection: the query or list to be transformed to JSON
-    :type collection: flasker.ext.orm.Query, list
-    :param model_id: model identifier. If specified, the parser will call
-      ``get`` on the query with this id.
-    :type model_id: dict
-    :param model_position: position of the model in the collection (1 indexed).
-    :type model_position: int
-    :param fast_count: if ``True``, will issue a separate count query to get
-      the total number of matches. Because of a limitation on how count is 
-      handled using subqueries, this results in much faster results. However
-      this can only be used if the initial ``collection`` argument is an
-      unfiltered query.
-    :type fast_count: bool
-    :param expect_one: can be used when ``model_id`` is specified to change the
-      return value of this method. Instead of returning a collection, it will
-      return the unique match found or raise an error otherwise.
-    :type expect_one: bool
-    :rtype: tuple or model
-
-    Returns a tuple ``(collection, match)``:
-
-    * ``collection`` is the filtered, sorted, offsetted, limited collection.
-    * ``match`` is a dictionary with two keys: ``total`` with the total number
-      of results from the filtered query and ``returned`` with the total number
-      of results for the filtered, offsetted and limited query.
-
-    """
-
-    if not collection:
-      return [], None
-
-    if model_id or model_position:
-
-      if model_id:
-        if not isinstance(collection, Query):
-          #TODO: adapt for lists
-          raise APIError(400, 'Only queries with model_id')
-        model_class = self._get_model_class(collection)
-        model_key = tuple(
-          model_id[k.name]
-          for k in class_mapper(model_class).primary_key
-        )
-        model = collection.get(model_key)
-        if expect_one:
-          if model:
-            return model
-          else:
-            raise APIError(400, 'Resource not found')
-        else:
-          collection = [model] if model else []
-
-      else:
-        position = int(model_position) - 1 # model_position is 1 indexed
-        if isinstance(collection, Query):
-          collection = collection.offset(position).limit(1).all()
-        else:
-          collection = collection[position:(position + 1)]
-
-      match = {'total': len(collection), 'returned': len(collection)}
-
-    else:
-      model = self._get_model_class(collection)
-
-      raw_filters = request.args.getlist('filter')
-      raw_sorts = request.args.getlist('sort')
-      offset = request.args.get('offset', 0, int)
-      limit = request.args.get('limit', self.options['default_limit'], int)
-      max_limit = self.options['max_limit']
-      if max_limit:
-        limit = min(limit, max_limit) if limit else max_limit
-
-      if isinstance(collection, Query):
-
-        sep = self.options['sep']
-
-        filters = []
-        for raw_filter in raw_filters:
-          try:
-            key, op, value = raw_filter.split(sep, 3)
-          except ValueError:
-            raise APIError(400, 'Invalid filter: %s' % raw_filter)
-          column = getattr(model, key, None)
-          if not column: # TODO check if is actual column
-            raise APIError(400, 'Invalid filter column: %s' % key)
-          if op == 'in':
-            filt = column.in_(value.split(','))
-          else:
-            try:
-              attr = filter(
-                lambda e: hasattr(column, e % op),
-                ['%s', '%s_', '__%s__']
-              )[0] % op
-            except IndexError:
-              raise APIError(400, 'Invalid filter operator: %s' % op)
-            if value == 'null':
-              value = None
-            filt = getattr(column, attr)(value)
-          filters.append(filt)
-
-        if fast_count:
-          count_query = model.c
-          for filt in filters:
-            collection = collection.filter(filt)
-            count_query = count_query.filter(filt)
-          match = {'total': count_query.scalar()}
-        else:
-          for filt in filters:
-            collection = collection.filter(filt)
-          match = {'total': collection.count()}
-
-        for raw_sort in raw_sorts:
-          try:
-            key, order = raw_sort.split(sep)
-          except ValueError:
-            raise APIError(400, 'Invalid sort: %s' % raw_sort)
-          if not order in ['asc', 'desc']:
-            raise APIError(400, 'Invalid sort order: %s' % order)
-          column = getattr(model, key, None)
-          if column:
-            collection = collection.order_by(getattr(column, order)())
-          else:
-            raise APIError(400, 'Invalid sort column: %s' % key)
-
-        if offset:
-          collection = collection.offset(offset)
-
-        if limit:
-          collection = collection.limit(limit)
-
-        match['returned'] = collection.count()
-
-      else:
-
-        if raw_filters or raw_sorts:
-          raise APIError(400, 'Filter and sorts not implemented for lists')
-
-        match = {'total': len(collection)}
-
-        if limit:
-          collection = collection[offset:(offset + limit)]
-        else:
-          collection = collection[offset:]
-
-        match['returned'] = len(collection)
-
-    return collection, match
-
-  def serialize(self, collection):
-    """Serializes a list or query to a dictionary.
-
-    :param collection: the collection to be serialized
-    :type collection: iterable
-    :param content_key: key to wrap content in
-    :type content_key: str
-    :rtype: dict
+    :param data: data. At this time, only instances, and lists of instances of
+      ``flasker.util.sqlalchemy.Model``, along with instances of 
+      ``flasker.util.sqlalchemy.Query`` are valid.
+    :type data: model or collection
+    :param data_key: key where the serialized data will go
+    :type data_key: str
+    :param meta_key: key where the metadata will go
+    :type meta_key: str
+    :param include_request: whether or not to include the issued request
+      information
+    :type include_request: bool
+    :param include_time: whether or not to include processing time
+    :type include_time: bool
+    :param include_matches: whether or not to include the total number of
+      results from the data (useful if ``data`` is a collection)
+    :type include_matches: bool
+    :rtype: Flask response
+    
+    Any keyword arguments will be included with the metadata.
     
-    The keyword arguments are ignored if ``content_key`` is ``None`` and
-    used to update the returned dictionary otherwise.
-
     """
-
-    expand = request.args.get('expand', self.options['expand'], int)
     depth = request.args.get('depth', self.options['default_depth'], int)
     max_depth = self.options['max_depth']
     if max_depth:
       depth = min(depth, max_depth)
 
-    return [
-      e.to_json(depth=depth, expand=expand)
-      for e in collection
-      if e
-    ]
+    start = time()
+
+    if isinstance(data, Model):
+      data = data.to_json(depth=depth)
+      match = 1
+    else:
+      col, matches = self._get_collection(data)
+      data = [e.to_json(depth=depth) for e in col if e]
+      match = {'total': matches, 'returned': len(data)}
+
+    rv = {data_key: data, meta_key: kwargs}
+
+    if include_matches:
+      rv[meta_key]['matches'] = match
+    if include_request:
+      rv[meta_key]['request'] = {
+        'base_url': request.base_url,
+        'method': request.method,
+        'values': request.values,
+      }
+    if include_time:
+      rv[meta_key]['parsing_time'] = time() - start
+
+    return jsonify(rv)
+
+  def _get_collection(self, collection):
+    """Parse query and return JSON.
+
+    :param collection: the query or list to be transformed to JSON
+    :type collection: flasker.ext.orm.Query, list
+    :rtype: tuple
+
+    Returns a tuple ``(collection, match)``:
+
+    * ``collection`` is the filtered, sorted, offsetted, limited collection.
+    * ``match`` is the total number of results from the filtered query 
+
+    """
+    model = self._get_model_class(collection)
+    raw_filters = request.args.getlist('filter')
+    raw_sorts = request.args.getlist('sort')
+    offset = request.args.get('offset', 0, int)
+    limit = request.args.get('limit', self.options['default_limit'], int)
+    max_limit = self.options['max_limit']
+    if max_limit:
+      limit = min(limit, max_limit) if limit else max_limit
+
+    if isinstance(collection, Query):
+
+      sep = self.options['sep']
+
+      for raw_filter in raw_filters:
+        try:
+          key, op, value = raw_filter.split(sep, 3)
+        except ValueError:
+          raise APIError(400, 'Invalid filter: %s' % raw_filter)
+        column = getattr(model, key, None)
+        if not column: # TODO check if is actual column
+          raise APIError(400, 'Invalid filter column: %s' % key)
+        if op == 'in':
+          filt = column.in_(value.split(','))
+        else:
+          try:
+            attr = filter(
+              lambda e: hasattr(column, e % op),
+              ['%s', '%s_', '__%s__']
+            )[0] % op
+          except IndexError:
+            raise APIError(400, 'Invalid filter operator: %s' % op)
+          if value == 'null':
+            value = None
+          filt = getattr(column, attr)(value)
+        collection = collection.filter(filt)
+
+      for raw_sort in raw_sorts:
+        try:
+          key, order = raw_sort.split(sep)
+        except ValueError:
+          raise APIError(400, 'Invalid sort: %s' % raw_sort)
+        if not order in ['asc', 'desc']:
+          raise APIError(400, 'Invalid sort order: %s' % order)
+        column = getattr(model, key, None)
+        if column:
+          collection = collection.order_by(getattr(column, order)())
+        else:
+          raise APIError(400, 'Invalid sort column: %s' % key)
+
+      matches = collection.fast_count()
+      if offset:
+        collection = collection.offset(offset)
+      if limit:
+        collection = collection.limit(limit)
+
+    else:
+      if raw_filters or raw_sorts:
+        raise APIError(400, 'Filter and sorts not implemented for lists')
+
+      matches = len(collection)
+      if limit:
+        collection = collection[offset:(offset + limit)]
+      else:
+        collection = collection[offset:]
+
+    return collection, matches
 
   def _get_model_class(self, collection):
     """Return corresponding model class from collection."""
