@@ -33,14 +33,12 @@ For convenience, both these variables are also available directly in the
 
 """
 
-from __future__ import absolute_import
-
 from logging import getLogger, NullHandler, StreamHandler, DEBUG
 from os.path import abspath, dirname, join, sep, split, splitext
 from sys import path
 from werkzeug.local import LocalProxy
 
-from .util import parse_config
+from .util.helpers import parse_config
 
 
 class ProjectImportError(Exception):
@@ -152,7 +150,7 @@ class Project(object):
 
     else:
 
-      if self.conf_path and conf_path != self.conf_path:
+      if self.conf_path and abspath(conf_path) != self.conf_path:
         raise ProjectImportError('Cannot instantiante projects for different '
                                  'configuration files in the same process.')
 
@@ -229,8 +227,8 @@ class Project(object):
         if not k in self.default_conf['FLASK']
       })
 
-      self._flask = flask_app
       self.logger.debug('flask app loaded')
+      self._flask = flask_app
     return self._flask
 
   @property
