@@ -3,7 +3,9 @@
 from tempfile import NamedTemporaryFile
 from nose.tools import eq_, raises
 
-from flasker.util import *
+from flasker.util.helpers import *
+from flasker.util.mixins import *
+from flasker.util.ndict import *
 
 
 def test_convert_auto():
@@ -129,27 +131,27 @@ class Test_Dict(object):
     ]
 
   def test_depth(self):
-    for example, depth in zip(self.examples, [3,2]):
-      eq_(Dict.depth(example['u']), depth)
-      eq_(Dict.depth(example['f']), 1)
+    for example, d in zip(self.examples, [3,2]):
+      eq_(depth(example['u']), d)
+      eq_(depth(example['f']), 1)
 
   def test_width(self):
-    for example, width in zip(self.examples, [4,2]):
+    for example, w in zip(self.examples, [4,2]):
       for v in example.values():
-        eq_(Dict.width(v), width)
+        eq_(width(v), w)
   
   def test_flatten(self):
     example = self.examples[0]
-    eq_(Dict.flatten(example['u']), example['f'])
+    eq_(flatten(example['u']), example['f'])
 
   def test_unflatten(self):
     for example in self.examples:
-      eq_(Dict.unflatten(example['f']), example['u'])
+      eq_(unflatten(example['f']), example['u'])
 
   def test_update(self):
     a = {'a': 1, 'b': {'c': 0}}
     b = {'a': 2, 'b': {'d': 1}}
-    c = Dict.update(a, b)
+    c = update(a, b)
     eq_(c, {'a': 2, 'b': {'c': 0, 'd': 1}})
     eq_(a, c)
     eq_(b, {'a': 2, 'b': {'d': 1}})
@@ -157,7 +159,7 @@ class Test_Dict(object):
   def test_update_with_copy(self):
     a = {'a': 1, 'b': {'c': 0}}
     b = {'a': 2, 'b': {'d': 1}}
-    c = Dict.update(a, b, copy=True)
+    c = update(a, b, copy=True)
     eq_(a, {'a': 1, 'b': {'c': 0}})
     eq_(b, {'a': 2, 'b': {'d': 1}})
   
